import { html, nothing } from 'lit-html';
import { ifDefined } from 'lit-html/directives/if-defined.js';
import { useState, useLayoutEffect, useEffect } from 'haunted';
import { v4 as uuid } from 'uuid';
import { createComponent } from '../../_lib/create-component';

function getScrollProperties(target) {
  return target
    ? {
        top: target.scrollTop > 0,
        bottom:
          Math.ceil(target.scrollTop) + target.clientHeight <
          target.scrollHeight
      }
    : {};
}

function tmplIndicator({ position }) {
  return html`
    <div class="scrollable__indicator scrollable__indicator--${position}">
      <div class="scrollable__indicator-inner"></div>
      <div class="scrollable__indicator-pinnacle"></div>
      <div class="scrollable__indicator-inner"></div>
    </div>
  `;
}

function ScrollableComponent(element) {
  const { props } = element;
  const {
    eventScrollToPlayingItem = false,
    eventScrollToTop = false,
    content
  } = props;
  const [scrollIndicators, setScrollIndicators] = useState({});

  useLayoutEffect(() => {
    window.requestAnimationFrame(() => {
      setTimeout(() => {
        handleIndicators();
      }, 500);
    });
  }, []);

  useEffect(() => {
    window.addEventListener('resize', handleIndicators);
    return () => {
      window.removeEventListener('resize', handleIndicators);
    };
  }, []);

  function handleScrollToPlayingItem(ev) {
    const { selectedItem } = ev.detail;
    selectedItem.scrollIntoView({ block: 'center' });
  }

  useEffect(() => {
    if (eventScrollToPlayingItem) {
      document.addEventListener(
        'scrollToPlayingItem',
        handleScrollToPlayingItem
      );
    }
    return () => {
      if (eventScrollToPlayingItem) {
        document.removeEventListener(
          'scrollToPlayingItem',
          handleScrollToPlayingItem
        );
      }
    };
  }, []);

  function handleScrollToTop() {
    element.querySelector('[data-scroll]').scrollTo({
      top: 0
    });
  }

  useEffect(() => {
    if (eventScrollToTop) {
      document.addEventListener('scrollToTop', handleScrollToTop);
    }
    return () => {
      if (eventScrollToTop) {
        document.removeEventListener('scrollToTop', handleScrollToTop);
      }
    };
  }, []);

  function handleIndicators() {
    setScrollIndicators(
      getScrollProperties(element.querySelector('[data-scroll]'))
    );
  }

  function handleScroll(ev) {
    setScrollIndicators(getScrollProperties(ev.currentTarget));
  }

  return html`
    <div class="scrollable">
      ${scrollIndicators?.top ? tmplIndicator({ position: 'top' }) : nothing}
      <div
        class="scrollable__inner"
        @scroll="${handleScroll}"
        data-scroll="true"
        tabindex="${ifDefined(
          scrollIndicators?.top || scrollIndicators?.bottom ? '0' : undefined
        )}"
      >
        ${content()}
      </div>
      ${scrollIndicators?.bottom
        ? tmplIndicator({ position: 'bottom' })
        : nothing}
    </div>
  `;
}

export function Scrollable(props) {
  createComponent({
    name: 'myth-scrollable',
    fnComponent: ScrollableComponent,
    useShadowDOM: false
  });

  const { extraClasses, id } = props;

  return html`
    <myth-scrollable
      class="${ifDefined(extraClasses || undefined)}"
      id="${id || `scrollable-${uuid()}`}"
      .props="${props}"
    ></myth-scrollable>
  `;
}
