import { html, nothing } from 'lit-html';
import { useState } from 'haunted';
import { classMap } from 'lit-html/directives/class-map.js';
import { handleTimeline } from './_lib/handle-timeline.js';
import { readable as formatTime } from 'mystic-format-time';
import { createComponent } from '../../_lib/create-component';

function getChapter({ targetTime, chapters }) {
  const selected = chapters.find(
    chapter =>
      targetTime >= chapter.start / 1000 && targetTime < chapter.end / 1000
  );
  return selected?.tags?.title || null;
}

function TimelineComponent(element) {
  const { props } = element;
  const { totalTime, currentTime, currentItem = {}, getMediaElement } = props;
  const [timePosition, setTimePosition] = useState(null);
  const chapters = currentItem?.metadata?.chapters || [];

  function handleTimelineIn(ev) {
    const target = ev.currentTarget;
    const leftPosition = ev.layerX;
    const targetWidth = target.offsetWidth;
    const targetTime = (leftPosition * totalTime) / targetWidth;
    setTimePosition({
      visual: formatTime(targetTime),
      chapter:
        chapters.length > 0 ? getChapter({ targetTime, chapters }) : null,
      left: leftPosition
    });
  }

  function handleTimelineOut() {
    setTimePosition(null);
  }

  function fnHandleTimeline(ev) {
    handleTimeline(ev, getMediaElement);
  }

  return html`
    <div class="timeline">
      <div
        class="timeline__track"
        @click="${fnHandleTimeline}"
        @keydown="${fnHandleTimeline}"
        @mousemove="${handleTimelineIn}"
        @mouseout="${handleTimelineOut}"
        role="progressbar"
        tabindex="0"
      >
        <div
          class="timeline__completed"
          style="width:${totalTime ? (100 / totalTime) * currentTime : 0}%;"
        ></div>
      </div>
      <div class="timeline__numbers">
        <div>${formatTime(currentTime)}</div>
        <div>${formatTime(totalTime)}</div>
      </div>
      ${chapters.map(chapter => {
        const limitClasses = {
          'timeline__chapter-limit': true,
          'timeline__chapter-limit--played': currentTime > chapter.start / 1000
        };
        const position = (100 / totalTime) * (chapter.start / 1000);
        return html`
          <div
            class="${classMap(limitClasses)}"
            style="--timeline--chapter-limit-position: ${position}%;"
          ></div>
        `;
      })}
      ${timePosition === null
        ? nothing
        : html`
            <div
              class="timeline__tooltip"
              style="--timeline-tooltip-position: ${timePosition.left}px;"
            >
              <div class="timeline__tooltip-inner">
                ${timePosition.chapter
                  ? html`
                      <span class="timeline__tooltip-chapter-title">
                        ${timePosition.chapter}
                      </span>
                    `
                  : nothing}
                <span class="timeline__tooltip-time">
                  ${timePosition.visual}
                </span>
              </div>
            </div>
          `}
    </div>
  `;
}

export function Timeline(props) {
  createComponent({
    name: 'mythical-player-timeline',
    fnComponent: TimelineComponent
  });

  return html`
    <mythical-player-timeline .props="${props}"></mythical-player-timeline>
  `;
}
